pub trait FromInput<In: SystemInput>: SystemInput {
// Required method
fn from_inner<'i>(inner: In::Inner<'i>) -> Self::Inner<'i>;
}Expand description
A type that may be constructed from the input of a System.
This is used to allow systems whose first parameter is a StaticSystemInput<In>
to take an In as input, and can be implemented for user types to allow
similar conversions.
Required Methods
Sourcefn from_inner<'i>(inner: In::Inner<'i>) -> Self::Inner<'i>
fn from_inner<'i>(inner: In::Inner<'i>) -> Self::Inner<'i>
Converts the system input’s inner representation into this type’s inner representation.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".