IntoResult

Trait IntoResult 

pub trait IntoResult<Out>: Sized {
    // Required method
    fn into_result(self) -> Result<Out, RunSystemError>;
}
A type that may be converted to the output of a System. This is used to allow systems to return either a plain value or a Result.

Required Methods

fn into_result(self) -> Result<Out, RunSystemError>

Converts this type into the system output type.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors